@@ -21,11 +21,21 @@ namespace timg {
 // Returns static string if successful or nullptr if not.
 const char* QueryBackgroundColor();
 
-// Query the terminal if it supports the Kitty graphics protocol.
-bool QueryHasKittyGraphics();
+enum class GraphicsProtocol {
+    kNone,
+    kIterm2,
+    kKitty,
+    kSixel,
+};
+struct TermGraphicsInfo {
+    GraphicsProtocol preferred_graphics;
+    bool known_broken_sixel_cursor_placement;
+    bool in_tmux;
+};
+
+// Query the terminal if and what graphics protocol it supports.
+TermGraphicsInfo QuerySupportedGraphicsProtocol();
 
-// Query if the terminal supports the iTerm2 graphics.
-bool QueryHasITerm2Graphics();
 }  // namespace timg
 
 #endif  // TIMG_TERM_QUERY_H
