@@ -27,9 +27,13 @@
 #include "terminal-canvas.h"
 #include "termutils.h"
 #include "thread-pool.h"
+#include "timg-help.h"
 #include "timg-time.h"
 #include "timg-version.h"
 #include "unicode-block-canvas.h"
+#ifdef WITH_TIMG_SIXEL
+#    include "sixel-canvas.h"
+#endif
 
 #ifdef WITH_TIMG_TERMINAL_QUERY
 #    include "term-query.h"
@@ -47,11 +51,15 @@
 
 #    include "image-display.h"
 #endif
+#ifdef WITH_TIMG_SIXEL
+#    include <sixel.h>
+#endif
 
 #include <errno.h>
 #include <fcntl.h>
 #include <getopt.h>
 #include <inttypes.h>
+#include <libswscale/swscale.h>  // Only needed for version.
 #include <math.h>
 #include <signal.h>
 #include <stdio.h>
@@ -66,6 +74,7 @@
 #include <future>
 #include <iostream>
 #include <memory>
+#include <mutex>
 #include <string>
 #include <thread>
 #include <vector>
@@ -99,6 +108,9 @@ enum class Pixelation {
     kQuarterBlock,
     kKittyGraphics,
     kiTerm2Graphics,
+#ifdef WITH_TIMG_SIXEL
+    kSixelGraphics,
+#endif
 };
 
 enum class ClearScreen {
@@ -112,6 +124,8 @@ namespace timg {
 struct PresentationOptions {
     // Rendering
     Pixelation pixelation         = Pixelation::kNotChosen;
+    bool sixel_cursor_workaround  = false;
+    bool tmux_workaround          = false;
     bool terminal_use_upper_block = false;
     bool use_256_color = false;  // For terminals that don't do 24 bit color
 
@@ -125,17 +139,16 @@ struct PresentationOptions {
     bool hide_cursor         = true;    // Hide cursor while emitting image
     ClearScreen clear_screen = ClearScreen::kNot;  // Clear between images ?
     Duration duration_between_images;  // How long to wait between images
+    Duration duration_for_row;         // Wait time for a grid row.
 };
 }  // namespace timg
 
 // Image sources; as future as they are being filled while we start presenting
 typedef std::vector<std::future<timg::ImageSource *>> LoadedImageSources;
 
-// Modern processors with some sort of hyperthreading don't seem to scale
-// much beyond their physical core count when doing image processing.
-// So just keep thread count at half what we get reported.
+// Use most cores that are available.
 static const int kDefaultThreadCount =
-    std::max(1U, std::thread::hardware_concurrency() / 2);
+    std::max(1, 3 * (int)std::thread::hardware_concurrency() / 4);
 
 volatile sig_atomic_t interrupt_received = 0;
 static void InterruptHandler(int signo) { interrupt_received = 1; }
@@ -147,36 +160,49 @@ static int usage(const char *progname, ExitCode exit_code, int width,
 #else
     static constexpr char kFileType[] = "image";
 #endif
+    const char *after_slash_progname = rindex(progname, '/');
+    progname = after_slash_progname ? after_slash_progname + 1 : progname;
+    const char *const default_title = getenv("TIMG_DEFAULT_TITLE");
     fprintf(stderr, "usage: %s [options] <%s> [<%s>...]\n", progname, kFileType,
             kFileType);
     fprintf(
         stderr,
-        "\e[1mOptions\e[0m:\n"
-        "\t-g<w>x<h>      : Output geometry in character cells. Partial "
-        "geometry\n"
-        "\t                 leaving out one value -g<w>x or -gx<h> is "
-        "possible,\n"
-        "\t                 the other value it then derived from the terminal "
-        "size.\n"
-        "\t                 Default derived from terminal size is %dx%d\n"
+        "\e[1mOptions\e[0m (most common first):\n"
         "\t-p<pixelation> : Pixelation: 'h' = half blocks    'q' = quarter "
         "blocks\n"
         "\t                             'k' = kitty graphics 'i' = iTerm2 "
         "graphics\n"
+#ifdef WITH_TIMG_SIXEL
+        "\t                             's' = sixel graphics\n"
+#endif
         "\t                 Default: Auto-detect graphics, otherwise "
         "'quarter'.\n"
-        "\t--compress     : Only for -pk or -pi: Compress image data. More\n"
-        "\t                 CPU use for timg, but less bandwidth needed.\n"
-        "\t-C, --center   : Center image horizontally.\n"
-        "\t-W, --fit-width: Scale to fit width of available space, even if it\n"
-        "\t                 exceeds height.\n"
-        "\t--grid=<cols>[x<rows>] : Arrange images in a grid (contact sheet).\n"
-        "\t-w<seconds>    : Wait time between images (default: 0.0).\n"
-        "\t-a             : Switch off anti aliasing (default: on).\n"
+        "\t--grid=<cols>[x<rows>] : Arrange images in a grid (\"contact "
+        "sheet\").\n"
+        "\t-C, --center   : Center image horizontally in available cell.\n"
+        "\t--title[=<fmt_str>]: Print title above each image. Accepts the "
+        "following\n"
+        "\t                 format parameters: %%f = full filename; %%b = "
+        "basename\n"
+        "\t                                    %%w = image width; %%h = "
+        "height\n"
+        "\t                                    %%D = internal decoder used\n"
+        "\t                 If no parameter is given, defaults to \"%%f\"\n"
+        "\t                 Env-var override with TIMG_DEFAULT_TITLE%s%s%s\n"
+        "\t-f<filelist>   : Read newline-separated list of image files to "
+        "show.\n"
+        "\t                 Relative filenames are relative to current "
+        "directory.\n"
+        "\t                 (-f and -F can be provided multiple times.)\n"
+        "\t-F<filelist>   : like -f, but relative filenames considered "
+        "relative\n"
+        "\t                 to the directory containing the filelist.\n"
         "\t-b<str>        : Background color to use behind alpha channel. "
         "Format\n"
-        "\t                 'yellow', '#rrggbb', 'auto' or 'none' (default "
-        "'auto').\n"
+        "\t                 color name like 'yellow', '#rrggbb', 'auto' or "
+        "'none'.\n"
+        "\t                'auto' is terminal background color. "
+        "(default 'auto').\n"
         "\t-B<str>        : Checkerboard pattern color to use on alpha.\n"
         "\t--pattern-size=<n> : Integer factor scale of the checkerboard "
         "pattern.\n"
@@ -187,15 +213,17 @@ static int usage(const char *progname, ExitCode exit_code, int width,
         "\t--rotate=<exif|off> : Rotate according to included exif "
         "orientation.\n"
         "\t                      or 'off'. Default: exif.\n"
-        "\t--clear        : Clear screen first. Optional argument 'every' "
-        "will\n"
-        "\t                 clear before every image (useful with -w)\n"
+        "\t-W, --fit-width: Scale to fit width of available space, even if it\n"
+        "\t                 exceeds height.\n"
         "\t-U, --upscale[=i]: Allow Upscaling. If an image is smaller than "
         "the\n"
         "\t                 available frame (e.g. an icon), enlarge it to "
         "fit.\n"
         "\t                 Optional parameter 'i' only enlarges in integer "
         "steps.\n"
+        "\t--clear        : Clear screen first. Optional argument 'every' "
+        "will\n"
+        "\t                 clear before every image (useful with -w)\n"
 #ifdef WITH_TIMG_VIDEO
         "\t-V             : Directly use Video subsystem. Don't probe image\n"
         "\t                 decoding first (useful, if you stream video from "
@@ -203,26 +231,28 @@ static int usage(const char *progname, ExitCode exit_code, int width,
         "\t-I             : Only  use Image subsystem. Don't attempt video "
         "decoding\n"
 #endif
-        "\t--title[=<fmt_str>]: Print title above each image. Accepts the "
-        "following\n"
-        "\t                 format parameters: %%f = full filename; %%b = "
-        "basename\n"
-        "\t                                    %%w = image width; %%h = "
-        "height\n"
-        "\t                                    %%D = internal decoder used\n"
-        "\t                 If no parameter is given, defaults to \"%%f\"\n"
-        "\t-F             : Print filename as title. Behaves like "
-        "--title=\"%%f\"\n"
-        "\t-f<filelist>   : Read newline-separated list of image files to "
-        "show.\n"
-        "\t                 (Can be provided multiple times.)\n"
+        "\t-w<seconds>    : Wait time between images (default: 0.0).\n"
+        "\t-wr<seconds>   : like above, but wait time between rows in grid.\n"
+        "\t-a             : Switch off anti-aliasing (default: on).\n"
+        "\t-g<w>x<h>      : Output geometry in character cells. Partial "
+        "geometry\n"
+        "\t                 leaving out one value -g<w>x or -gx<h> is "
+        "possible,\n"
+        "\t                 the other value it then derived from the terminal "
+        "size.\n"
+        "\t                 Default derived from terminal size is %dx%d\n"
         "\t-o<outfile>    : Write to <outfile> instead of stdout.\n"
         "\t-E             : Don't hide the cursor while showing images.\n"
+        "\t--compress[=level]: Only for -pk or -pi: Compress image data. More\n"
+        "\t                 CPU use, but less used bandwidth. (default: 1)\n"
         "\t--threads=<n>  : Run image decoding in parallel with n threads\n"
-        "\t                 (Default %d, half #cores on this machine)\n"
+        "\t                 (Default %d, 3/4 #cores on this machine)\n"
         "\t--color8       : Choose 8 bit color mode for -ph or -pq\n"
-        "\t--version      : Print version and exit.\n"
-        "\t-h, --help     : Print this help and exit.\n"
+        "\t--version      : Print detailed version including used libraries.\n"
+        "\t                 (%s)\n"
+        "\t--verbose      : Print some stats after images shown.\n"
+        "\t-h             : Print this help and exit.\n"
+        "\t--help         : Page through detailed manpage-like help and exit.\n"
 
         "\n  \e[1mScrolling\e[0m\n"
         "\t--scroll=[<ms>]       : Scroll horizontally (optionally: delay ms "
@@ -242,13 +272,17 @@ static int usage(const char *progname, ExitCode exit_code, int width,
         "\t--frame-offset=<num>: Start animation/video at this frame\n"
         "\t-t<seconds>   : Stop after this time, independent of --loops or "
         "--frames\n",
-        width, height, kDefaultThreadCount);
+        default_title ? "='" : "", default_title ? default_title : "",
+        default_title ? "'" : "", width, height, kDefaultThreadCount,
+        TIMG_VERSION);
     return (int)exit_code;
 }
 
-// Read list of filenames from newline separated file. Non-absolute files
-// are resolved relative to the filelist_file
+// Read list of filenames from newline separated file.
+// Non-absolute files are resolved relative to the filelist_file or
+// relative to currnet working directory.
 bool AppendToFileList(const std::string &filelist_file,
+                      bool consider_relative_to_filelist,
                       std::vector<std::string> *filelist) {
     std::ifstream filelist_stream(
         filelist_file == "-" ? "/dev/stdin" : filelist_file, std::ifstream::in);
@@ -262,7 +296,10 @@ bool AppendToFileList(const std::string &filelist_file,
     std::string filename;
     for (std::string filename; std::getline(filelist_stream, filename); /**/) {
         if (filename.empty()) continue;
-        if (filename[0] != '/' && !prefix.empty()) filename.insert(0, prefix);
+        if (consider_relative_to_filelist &&  //
+            filename[0] != '/' && !prefix.empty()) {
+            filename.insert(0, prefix);
+        }
         filelist->push_back(filename);
     }
     return true;
@@ -273,14 +310,29 @@ static void PresentImages(LoadedImageSources *loaded_sources,
                           const timg::PresentationOptions &present,
                           timg::BufferedWriteSequencer *sequencer,
                           bool *any_animations_seen) {
+    using timg::ThreadPool;
+    std::unique_ptr<ThreadPool> compression_pool;
     std::unique_ptr<TerminalCanvas> canvas;
     switch (present.pixelation) {
     case Pixelation::kKittyGraphics:
-        canvas.reset(new KittyGraphicsCanvas(sequencer, display_opts));
+        compression_pool.reset(new ThreadPool(sequencer->max_queue_len() + 1));
+        canvas.reset(new KittyGraphicsCanvas(sequencer, compression_pool.get(),
+                                             present.tmux_workaround,
+                                             display_opts));
         break;
     case Pixelation::kiTerm2Graphics:
-        canvas.reset(new ITerm2GraphicsCanvas(sequencer, display_opts));
+        compression_pool.reset(new ThreadPool(sequencer->max_queue_len() + 1));
+        canvas.reset(new ITerm2GraphicsCanvas(sequencer, compression_pool.get(),
+                                              display_opts));
         break;
+#ifdef WITH_TIMG_SIXEL
+    case Pixelation::kSixelGraphics:
+        compression_pool.reset(new ThreadPool(sequencer->max_queue_len() + 1));
+        canvas.reset(new timg::SixelCanvas(sequencer, compression_pool.get(),
+                                           present.sixel_cursor_workaround,
+                                           display_opts));
+        break;
+#endif
     case Pixelation::kHalfBlock:
     case Pixelation::kQuarterBlock:
     case Pixelation::kNotChosen:  // Should not happen.
@@ -290,7 +342,8 @@ static void PresentImages(LoadedImageSources *loaded_sources,
     }
 
     auto renderer = timg::Renderer::Create(
-        canvas.get(), display_opts, present.grid_cols, present.grid_rows);
+        canvas.get(), display_opts, present.grid_cols, present.grid_rows,
+        present.duration_between_images, present.duration_for_row);
 
     // Things to do before and after we show an image. Our goal is to keep
     // the terminal always in a good state (cursor on!) while also reacting
@@ -330,11 +383,10 @@ static void PresentImages(LoadedImageSources *loaded_sources,
                            renderer->render_cb(
                                source->FormatTitle(display_opts.title_format)));
         after_image_show();
-        if (!present.duration_between_images.is_zero()) {
-            (Time::Now() + present.duration_between_images).WaitUntil();
-        }
+        renderer->MaybeWaitBetweenImageSources();
         is_first = false;
     }
+    sequencer->Flush();
 }
 
 int main(int argc, char *argv[]) {
@@ -366,11 +418,19 @@ int main(int argc, char *argv[]) {
     int geometry_height       = (term.rows - 2);
     bool debug_no_frame_delay = false;
 
-    // Convenience predicate: pixelation sending high-res images, no blocks.
-    const auto is_pixel_direct_p = [](Pixelation p) {
+    // Convenience predicates: pixelation sending high-res images, no blocks.
+    const auto is_pixel_direct_with_alpha = [](Pixelation p) {
         return p == Pixelation::kKittyGraphics ||
                p == Pixelation::kiTerm2Graphics;
     };
+    const auto is_pixel_direct_p = [](Pixelation p) {
+        return p == Pixelation::kKittyGraphics ||
+               p == Pixelation::kiTerm2Graphics
+#ifdef WITH_TIMG_SIXEL
+               || p == Pixelation::kSixelGraphics
+#endif
+            ;
+    };
 
     enum LongOptionIds {
         OPT_CLEAR_SCREEN = 1000,
@@ -383,8 +443,10 @@ int main(int argc, char *argv[]) {
         OPT_PATTERN_SIZE,
         OPT_ROTATE,
         OPT_THREADS,
+        OPT_TITLE,
         OPT_VERBOSE,
         OPT_VERSION,
+        OPT_MANPAGE_HELP,
         OPT_AUTO_CROP,
         OPT_SCROLL,
     };
@@ -396,21 +458,21 @@ int main(int argc, char *argv[]) {
         {"center",               no_argument,       NULL, 'C'               },
         {"clear",                optional_argument, NULL, OPT_CLEAR_SCREEN  },
         {"color8",               no_argument,       NULL, OPT_COLOR_256     },
-        {"compress",             no_argument,       NULL, OPT_COMPRESS_PIXEL},
+        {"compress",             optional_argument, NULL, OPT_COMPRESS_PIXEL},
         {"delta-move",           required_argument, NULL, 'd'               },
         {"debug-no-frame-delay", no_argument,       NULL, OPT_NO_FRAME_DELAY},
         {"frame-offset",         required_argument, NULL, OPT_FRAME_OFFSET  },
         {"fit-width",            no_argument,       NULL, 'W'               },
         {"frames",               required_argument, NULL, OPT_FRAME_COUNT   },
         {"grid",                 required_argument, NULL, OPT_GRID          },
-        {"help",                 no_argument,       NULL, 'h'               },
+        {"help",                 no_argument,       NULL, OPT_MANPAGE_HELP  },
         {"loops",                optional_argument, NULL, 'c'               },
         {"pattern-size",         required_argument, NULL, OPT_PATTERN_SIZE  },
         {"pixelation",           required_argument, NULL, 'p'               },
         {"rotate",               required_argument, NULL, OPT_ROTATE        },
         {"scroll",               optional_argument, NULL, OPT_SCROLL        },
         {"threads",              required_argument, NULL, OPT_THREADS       },
-        {"title",                optional_argument, NULL, 'F'               },
+        {"title",                optional_argument, NULL, OPT_TITLE         },
         {"upscale",              optional_argument, NULL, 'U'               },
         {"verbose",              no_argument,       NULL, OPT_VERBOSE       },
         {"version",              no_argument,       NULL, OPT_VERSION       },
@@ -419,7 +481,7 @@ int main(int argc, char *argv[]) {
 
     int opt;
     int option_index = 0;
-    while ((opt = getopt_long(argc, argv, "vg:w:t:c:f:b:B:hCFEd:UWaVIo:f:p:",
+    while ((opt = getopt_long(argc, argv, "vg:w:t:c:f:b:B:hCF:Ed:UWaVIo:f:p:h",
                               long_options, &option_index)) != -1) {
         switch (opt) {
         case 'g':
@@ -433,8 +495,14 @@ int main(int argc, char *argv[]) {
             }
             break;
         case 'w':
-            present.duration_between_images =
-                Duration::Millis(roundf(atof(optarg) * 1000.0f));
+            if (optarg[0] == 'r') {
+                present.duration_for_row =
+                    Duration::Millis(roundf(atof(optarg + 1) * 1000.0f));
+            }
+            else {
+                present.duration_between_images =
+                    Duration::Millis(roundf(atof(optarg) * 1000.0f));
+            }
             break;
         case 't':
             present.duration_per_image =
@@ -542,8 +610,10 @@ int main(int argc, char *argv[]) {
             }
             break;
         case 'F':
-            display_opts.show_title = !display_opts.show_title;
-            if (optarg) display_opts.title_format = optarg;
+            if (!AppendToFileList(optarg, true, &filelist)) {
+                return usage(argv[0], ExitCode::kFilelistProblem,
+                             geometry_width, geometry_height);
+            }
             break;
         case 'E': present.hide_cursor = false; break;
         case 'W': display_opts.fill_width = true; break;
@@ -551,7 +621,7 @@ int main(int argc, char *argv[]) {
             fprintf(stderr,
                     "timg " TIMG_VERSION
                     " <https://timg.sh/>\n"
-                    "Copyright (c) 2016..2022 Henner Zeller. "
+                    "Copyright (c) 2016..2023 Henner Zeller. "
                     "This program is free software; license GPL 2.0.\n\n");
 #ifdef WITH_TIMG_GRPAPHICSMAGICK
             fprintf(stderr, "Image decoding %s\n",
@@ -565,16 +635,34 @@ int main(int argc, char *argv[]) {
             fprintf(stderr, "Turbo JPEG\n");
 #endif
 #ifdef WITH_TIMG_QOI
-            fprintf(stderr, "QOI Image reader\n");
+            fprintf(stderr, "QOI image loading\n");
 #endif
 #ifdef WITH_TIMG_STB
-            fprintf(stderr, "STB image loading fallback\n");
+            fprintf(stderr,
+                    "STB image loading"
+#    ifdef WITH_TIMG_GRPAPHICSMAGICK
+                    // If we have graphics magic, that will take images first,
+                    // so STB will only really be called as fallback.
+                    " fallback"
+#    endif
+                    "\n");
 #endif
+            fprintf(stderr, "swscale %s\n", AV_STRINGIFY(LIBSWSCALE_VERSION));
 #ifdef WITH_TIMG_VIDEO
             fprintf(stderr, "Video decoding %s\n",
                     timg::VideoLoader::VersionInfo());
+#endif
+            fprintf(stderr,
+                    "Half, quarter, iterm2, and kitty graphics output: "
+                    "timg builtin.\n");
+#ifdef WITH_TIMG_SIXEL
+            fprintf(stderr, "Libsixel version %s\n", LIBSIXEL_VERSION);
 #endif
             return 0;
+        case OPT_TITLE:
+            display_opts.show_title = !display_opts.show_title;
+            if (optarg) display_opts.title_format = optarg;
+            break;
         case 'o':
             output_fd = open(optarg, O_WRONLY | O_CREAT | O_TRUNC, 0664);
             if (output_fd < 0) {
@@ -584,7 +672,7 @@ int main(int argc, char *argv[]) {
             }
             break;
         case 'f':
-            if (!AppendToFileList(optarg, &filelist)) {
+            if (!AppendToFileList(optarg, false, &filelist)) {
                 return usage(argv[0], ExitCode::kFilelistProblem,
                              geometry_width, geometry_height);
             }
@@ -595,14 +683,31 @@ int main(int argc, char *argv[]) {
             case 'q': present.pixelation = Pixelation::kQuarterBlock; break;
             case 'k': present.pixelation = Pixelation::kKittyGraphics; break;
             case 'i': present.pixelation = Pixelation::kiTerm2Graphics; break;
+#ifdef WITH_TIMG_SIXEL
+            case 's': present.pixelation = Pixelation::kSixelGraphics; break;
+#endif
+            default:
+                fprintf(stderr, "Unknown --pixelation/-p parameter '%s'\n",
+                        optarg);
             }
             break;
         case OPT_COMPRESS_PIXEL:
-            display_opts.compress_pixel_format = true;
+            if (optarg) {
+                int level = atoi(optarg);
+                level     = (level >= 0 && level <= 9) ? level : 1;
+                display_opts.compress_pixel_level = level;
+            }
+            else {
+                display_opts.compress_pixel_level = 1;
+            }
             break;
         case OPT_COLOR_256: present.use_256_color = true; break;
         case OPT_VERBOSE: verbose = true; break;
         case OPT_NO_FRAME_DELAY: debug_no_frame_delay = true; break;
+        case OPT_MANPAGE_HELP:
+            InvokeHelpPager();
+            return 0;
+            break;
         case 'h':
         default:
             return usage(
@@ -649,7 +754,7 @@ int main(int argc, char *argv[]) {
         display_opts.cell_x_px = 9;  // Make up some typical values.
         display_opts.cell_y_px = 18;
         // hterm does _not_ support PNM, always convert to PNG.
-        display_opts.compress_pixel_format = true;
+        display_opts.compress_pixel_level = 1;
         // Because we don't know how much to move up and right. Also, hterm
         // does not seem to place an image in X-direction in the first place.
         present.grid_cols = 1;
@@ -659,25 +764,57 @@ int main(int argc, char *argv[]) {
     if (present.pixelation == Pixelation::kNotChosen) {
         present.pixelation = Pixelation::kQuarterBlock;  // Good default.
 #ifdef WITH_TIMG_TERMINAL_QUERY
-        // Konsole has the bad behaviour that it does not absorb the kitty
-        // graphics query but spills it on the screen. "Luckily", Konsole has
-        // another bug not returning the window pixel size, so we can use that
-        // to avoid the query :)
         if (term.font_width_px > 0 && term.font_height_px > 0) {
-            if (timg::QueryHasITerm2Graphics())
+            auto graphics_info      = timg::QuerySupportedGraphicsProtocol();
+            present.tmux_workaround = graphics_info.in_tmux;
+            switch (graphics_info.preferred_graphics) {
+            case timg::GraphicsProtocol::kIterm2:
                 present.pixelation = Pixelation::kiTerm2Graphics;
-            else if (timg::QueryHasKittyGraphics())
+                break;
+            case timg::GraphicsProtocol::kKitty:
                 present.pixelation = Pixelation::kKittyGraphics;
+                break;
+            case timg::GraphicsProtocol::kSixel:
+#    ifdef WITH_TIMG_SIXEL
+                present.pixelation = Pixelation::kSixelGraphics;
+                present.sixel_cursor_workaround =
+                    graphics_info.known_broken_sixel_cursor_placement;
+#    else
+                present.pixelation = Pixelation::kQuarterBlock;
+#    endif
+                break;
+            case timg::GraphicsProtocol::kNone: break;
+            }
         }
 #endif
     }
+    else if (present.pixelation == Pixelation::kKittyGraphics) {
+        // If the user manually chooses kitty, we still need to know if in tmux
+        auto graphics_info      = timg::QuerySupportedGraphicsProtocol();
+        present.tmux_workaround = graphics_info.in_tmux;
+    }
+#if defined(WITH_TIMG_SIXEL) && defined(WITH_TIMG_TERMINAL_QUERY)
+    // If the user manually choose sixel, we still can't avoid a terminal
+    // query, as we have to figure out if it has a broken cursor implementation.
+    else if (present.pixelation == Pixelation::kSixelGraphics) {
+        auto graphics_info = timg::QuerySupportedGraphicsProtocol();
+        present.sixel_cursor_workaround =
+            graphics_info.known_broken_sixel_cursor_placement;
+    }
+#endif
 
-    // If 'none' is chosen for background color, then using the
-    // PNG compression with alpha channels gives us compositing on client side
+    // The high-res image terminals provide alpha-blending, no need to
+    // query the terminal color for 'auto'
+    if (is_pixel_direct_with_alpha(present.pixelation) &&
+        (strcasecmp(bg_color, "auto") == 0)) {
+        bg_color = "none";
+    }
+
+    // If 'none' is chosen for background color, then emitting the
+    // PNG with alpha channels gives us compositing on client side.
     if (is_pixel_direct_p(present.pixelation) &&
-        strcasecmp(bg_color, "none") == 0) {
-        display_opts.compress_pixel_format = true;
-        display_opts.local_alpha_handling  = false;
+        (strcasecmp(bg_color, "none") == 0)) {
+        display_opts.local_alpha_handling = false;
     }
 
     // If we're using block graphics, we might need to adapt the aspect ratio
@@ -700,6 +837,9 @@ int main(int argc, char *argv[]) {
         display_opts.cell_x_px = 2;
         display_opts.cell_y_px = 2;
         break;
+#ifdef WITH_TIMG_SIXEL
+    case Pixelation::kSixelGraphics:
+#endif
     case Pixelation::kKittyGraphics:
     case Pixelation::kiTerm2Graphics:
         if (term.font_width_px > 0) display_opts.cell_x_px = term.font_width_px;
@@ -798,8 +938,8 @@ int main(int argc, char *argv[]) {
 
     ExitCode exit_code = ExitCode::kSuccess;
 
-    // Only print errors if we got one explicit filename.
-    const bool print_errors = (filelist.size() == 1);
+    std::mutex errors_lock;  // Collect any errors to display later.
+    std::deque<std::string> errors;
 
     // Async image loading, preparing them in a thread pool
     LoadedImageSources loaded_sources;
@@ -807,18 +947,28 @@ int main(int argc, char *argv[]) {
         if (interrupt_received) break;
         std::function<timg::ImageSource *()> f =
             [filename, frame_offset, max_frames, do_img_loading, do_vid_loading,
-             print_errors, &display_opts, &exit_code]() -> timg::ImageSource * {
+             &display_opts, &exit_code, &errors_lock,
+             &errors]() -> timg::ImageSource * {
             if (interrupt_received) return nullptr;
+            // TODO: after switch to c++17, use variant in return ?
+            std::string err;
             ImageSource *result = ImageSource::Create(
                 filename, display_opts, frame_offset, max_frames,
-                do_img_loading, do_vid_loading, print_errors);
-            if (!result) exit_code = ExitCode::kImageReadError;
+                do_img_loading, do_vid_loading, &err);
+            if (!result) {
+                std::unique_lock<std::mutex> l(errors_lock);
+                exit_code = ExitCode::kImageReadError;
+                if (!err.empty()) errors.push_back(err);
+            }
             return result;
         };
         loaded_sources.push_back(pool->ExecAsync(f));
     }
 
-    static constexpr int kAsyncWriteQueueSize = 3;
+    // The aync write queue (BufferedWriteSequencer) lines up the next
+    // buffers to be emitted.
+    static constexpr int kAsyncWriteQueueSize = 4;
+
     // Since Unicode blocks emit differences, we can't skip frames in output.
     // TODO: should probably better ask the canvas directly instead.
     const bool buffer_allow_skipping = (display_opts.allow_frame_skipping &&
@@ -829,9 +979,19 @@ int main(int argc, char *argv[]) {
     const Time start_show = Time::Now();
     PresentImages(&loaded_sources, display_opts, present, &sequencer,
                   &cell_size_warning_needed);
-    sequencer.Flush();
     const Time end_show = Time::Now();
 
+    // Error messages have been collected to not clutter the image output
+    // when they happen. Emit them now.
+    int max_errors_to_show = 4;
+    for (const std::string &err : errors) {
+        fprintf(stderr, "%s\n", err.c_str());
+        if (--max_errors_to_show == 0) break;
+    }
+    if (!max_errors_to_show) {
+        fprintf(stderr, "... total of %d errors\n", (int)errors.size());
+    }
+
     if (interrupt_received) {
         // Even though we completed the write, some terminals sometimes seem
         // to get messed up, maybe interrupted escape sequence ?
@@ -852,13 +1012,45 @@ int main(int argc, char *argv[]) {
                     "\t->File a feature request with the terminal "
                     "emulator program you use\n");
         }
-        fprintf(stderr, "Active Geometry: %dx%d\n", geometry_width,
+        fprintf(stderr, "Active Geometry: %dx%d", geometry_width,
                 geometry_height);
+        if (is_pixel_direct_p(present.pixelation) &&  //
+            term.font_width_px > 0 && term.font_height_px > 0) {
+            fprintf(stderr, "; canvas-pixels: %dx%d",
+                    geometry_width * term.font_width_px,
+                    geometry_height * term.font_height_px);
+        }
+        fprintf(stderr, "\n");
+
+        const char *pixelation = "";
+        switch (present.pixelation) {
+        case Pixelation::kHalfBlock: pixelation = "half block"; break;
+        case Pixelation::kQuarterBlock: pixelation = "quarter block"; break;
+        case Pixelation::kKittyGraphics: pixelation = "kitty graphics"; break;
+        case Pixelation::kiTerm2Graphics: pixelation = "iterm2 graphics"; break;
+#ifdef WITH_TIMG_SIXEL
+        case Pixelation::kSixelGraphics: pixelation = "sixel graphics"; break;
+#endif
+        case Pixelation::kNotChosen: pixelation = "(none)"; break;
+        }
+        fprintf(stderr, "Using %s pixels", pixelation);
+#ifdef WITH_TIMG_SIXEL
+        if (present.pixelation == Pixelation::kSixelGraphics) {
+            if (present.sixel_cursor_workaround) {
+                fprintf(stderr, " (with cursor placment workaround)");
+            }
+            else {
+                fprintf(stderr, " (with default cursor placement)");
+            }
+        }
+#endif
+        fprintf(stderr, ".\n");
         const rgba_t bg = display_opts.bgcolor_getter();
-        fprintf(stderr,
-                "Background color for transparency '%s', effective "
-                "RGB #%02x%02x%02x\n",
-                bg_color, bg.r, bg.g, bg.b);
+        fprintf(stderr, "Background color for transparency '%s'", bg_color);
+        if (strcasecmp(bg_color, "none") != 0) {
+            fprintf(stderr, "; effective RGB #%02x%02x%02x", bg.r, bg.g, bg.b);
+        }
+        fprintf(stderr, "\n");
         const rgba_t checker_bg = display_opts.bg_pattern_color;
         if (checker_bg.a == 0xff) {
             fprintf(stderr,
@@ -877,7 +1069,7 @@ int main(int argc, char *argv[]) {
                 timg::HumanReadableByteValue(written_bytes / d).c_str(),
                 sequencer.frames_total());
         // Only show FPS if we have one video or animation
-        if (filelist.size() == 1 && sequencer.frames_total() > 100) {
+        if (filelist.size() == 1 && sequencer.frames_total() > 50) {
             fprintf(stderr, "; %.1ffps", sequencer.frames_total() / d);
         }
         if (display_opts.allow_frame_skipping && sequencer.frames_total() > 0) {
